import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:badges/badges.dart' as badges;
import 'package:overlay_support/overlay_support.dart';

import 'package:sudoku/ui/layout/game.dart';
import 'package:sudoku/ui/layout/parameters.dart';
import 'package:sudoku/provider/data.dart';
import 'package:sudoku/utils/game_utils.dart';

class Home extends StatefulWidget {
  const Home({super.key});

  @override
  HomeState createState() => HomeState();
}

class HomeState extends State<Home> {
  @override
  void initState() {
    super.initState();

    Data myProvider = Provider.of<Data>(context, listen: false);
    myProvider.initParametersValues();
    myProvider.loadCurrentSavedState();
  }

  List<String> getImagesAssets(Data myProvider) {
    final List<String> assets = [];

    final List<String> gameImages = [
      'button_back',
      'button_help',
      'button_show_conflicts',
      'button_start',
      'game_win',
      'placeholder',
      'cell_empty'
    ];
    for (String level in myProvider.availableLevelValues) {
      gameImages.add('level_$level');
    }
    for (String size in myProvider.availableSizeValues) {
      gameImages.add('size_$size');
    }

    for (String image in gameImages) {
      assets.add('${'assets/icons/$image'}.png');
    }

    List<String> skinImages = [];
    for (int value = 1; value <= 16; value++) {
      skinImages.add(value.toString());
    }

    for (String skin in myProvider.availableSkinValues) {
      for (String image in skinImages) {
        assets.add('${'${'assets/skins/$skin'}_$image'}.png');
      }
    }

    return assets;
  }

  @override
  Widget build(BuildContext context) {
    final Data myProvider = Provider.of<Data>(context);

    if (!myProvider.assetsPreloaded) {
      final List<String> assets = getImagesAssets(myProvider);
      for (String asset in assets) {
        precacheImage(AssetImage(asset), context);
      }
      myProvider.updateAssetsPreloaded(true);
    }

    final List<Widget> menuActions = [];

    if (myProvider.gameIsRunning) {
      menuActions.add(TextButton(
        child: Container(
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(4),
            border: Border.all(
              color: Colors.white,
              width: 3,
            ),
          ),
          child: const Image(
            image: AssetImage('assets/icons/button_back.png'),
            fit: BoxFit.fill,
          ),
        ),
        onPressed: () => toast('Long press to quit game...'),
        onLongPress: () => GameUtils.quitGame(myProvider),
      ));
      menuActions.add(const Spacer(flex: 6));
      menuActions.add(TextButton(
        child: Container(
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(4),
            border: Border.all(
              color: Colors.white,
              width: 3,
            ),
          ),
          child: badges.Badge(
            showBadge: myProvider.givenTipsCount == 0 ? false : true,
            badgeStyle: badges.BadgeStyle(
              badgeColor: myProvider.givenTipsCount < 10
                  ? Colors.green
                  : myProvider.givenTipsCount < 20
                      ? Colors.orange
                      : Colors.red,
            ),
            badgeContent: Text(
              myProvider.givenTipsCount == 0 ? '' : myProvider.givenTipsCount.toString(),
              style: const TextStyle(color: Colors.white),
            ),
            child: Container(
              padding: EdgeInsets.all(myProvider.givenTipsCountEnableCountdown / 4),
              child: const Image(
                image: AssetImage('assets/icons/button_help.png'),
                fit: BoxFit.fill,
              ),
            ),
          ),
        ),
        onPressed: () => myProvider.canGiveTip() ? GameUtils.showTip(myProvider) : null,
      ));
      menuActions.add(const Spacer());
      menuActions.add(TextButton(
        child: Container(
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(4),
            border: Border.all(
              color: myProvider.showConflicts ? Colors.blue : Colors.white,
              width: 3,
            ),
          ),
          child: const Image(
            image: AssetImage('assets/icons/button_show_conflicts.png'),
            fit: BoxFit.fill,
          ),
        ),
        onPressed: () {
          myProvider.toggleShowConflicts();
        },
      ));
    }

    return Scaffold(
      appBar: AppBar(
        actions: menuActions,
      ),
      body: SafeArea(
        child: Center(
          child: myProvider.gameIsRunning
              ? Game(myProvider: myProvider)
              : Parameters(myProvider: myProvider),
        ),
      ),
    );
  }
}
