import 'dart:math';

import 'package:flutter/material.dart';

import 'package:sudoku/entities/cell.dart';
import 'package:sudoku/provider/data.dart';

class SelectCellValueBar extends StatelessWidget {
  const SelectCellValueBar({super.key, required this.myProvider});

  final Data myProvider;

  @override
  Widget build(BuildContext context) {
    final Board cells = myProvider.board;

    final bool isCellSelected =
        (myProvider.selectedCellCol != null && myProvider.selectedCellRow != null);
    final bool isUpdatableCellSelected = isCellSelected
        ? !cells[myProvider.selectedCellRow ?? 0][myProvider.selectedCellCol ?? 0].isFixed
        : false;
    final int maxValue = myProvider.blockSizeHorizontal * myProvider.blockSizeVertical;

    const int maxItemsPerLine = 10;
    final int linesCount = (maxValue / maxItemsPerLine).floor() + 1;
    final int itemsCountPerLine = min(maxItemsPerLine, maxValue + 1);

    return Container(
      margin: const EdgeInsets.all(2),
      padding: const EdgeInsets.all(2),
      child: Table(
        defaultColumnWidth: const IntrinsicColumnWidth(),
        children: [
          for (int lineIndex = 0; lineIndex < linesCount; lineIndex++)
            TableRow(
              children: [
                for (int value = lineIndex * itemsCountPerLine;
                    value < (lineIndex + 1) * itemsCountPerLine;
                    value++)
                  Column(
                    children: [
                      Cell(
                        row: 1,
                        col: value,
                        value: isUpdatableCellSelected ? (value <= maxValue ? value : -1) : -1,
                        isFixed: false,
                      ).widgetUpdateValue(myProvider)
                    ],
                  ),
              ],
            ),
        ],
      ),
    );
  }
}
